@@ -12,6 +12,22 @@
 #include <KSharedConfig>
 #include <canberra.h>
 
+CalibrationTool::CalibrationTool()
+{
+    m_resetTimer.setInterval(1000);
+    connect(&m_resetTimer, &QTimer::timeout, this, [this] {
+        m_resetCountdown -= 1;
+        Q_EMIT resetSecondsLeftChanged();
+        if (m_resetCountdown > 0) {
+            m_resetTimer.start();
+        } else {
+            reset();
+            m_resetTimer.stop();
+            Q_EMIT resetFromSaved();
+        }
+    });
+}
+
 void CalibrationTool::setWidth(const float width)
 {
     if (m_width != width) {
@@ -38,9 +54,9 @@ float CalibrationTool::height()
     return m_height;
 }
 
-bool CalibrationTool::finishedCalibration() const
+CalibrationTool::State CalibrationTool::state() const
 {
-    return m_finishedCalibration;
+    return m_state;
 }
 
 int CalibrationTool::currentTarget() const
@@ -48,21 +64,34 @@ int CalibrationTool::currentTarget() const
     return m_calibratedTargets;
 }
 
+int CalibrationTool::resetSecondsLeft() const
+{
+    return m_resetCountdown;
+}
+
 void CalibrationTool::calibrate(const double touchX, const double touchY, const double screenX, const double screenY)
 {
     Q_ASSERT(m_calibratedTargets >= 0 && m_calibratedTargets < 4);
     if (m_calibratedTargets < 0 || m_calibratedTargets >= 4) {
         return;
     }
+    // Testing mode shouldn't call this function
+    Q_ASSERT(m_state != State::Testing);
+    if (m_state == State::Testing) {
+        return;
+    }
 
-    m_screenPoints[m_calibratedTargets] = {screenX, screenY};
-    m_touchPoints[m_calibratedTargets] = {touchX, touchY};
+    if (m_state == State::Calibrating) {
+        m_screenPoints[m_calibratedTargets] = {screenX, screenY};
+        m_touchPoints[m_calibratedTargets] = {touchX, touchY};
+    }
+
+    playSound(QStringLiteral("completion-partial"));
 
     m_calibratedTargets++;
     Q_EMIT currentTargetChanged();
 
     checkIfFinished();
-    playSound(QStringLiteral("completion-partial"));
 }
 
 void CalibrationTool::setCalibrationMatrix(InputDevice *device, const QMatrix4x4 &matrix)
@@ -83,8 +112,8 @@ void CalibrationTool::reset()
     m_calibratedTargets = 0;
     Q_EMIT currentTargetChanged();
 
-    m_finishedCalibration = false;
-    Q_EMIT finishedCalibrationChanged();
+    m_state = State::Calibrating;
+    Q_EMIT stateChanged();
 }
 
 void ca_finish_callback(ca_context *c, uint32_t id, int error_code, void *userdata)
@@ -124,81 +153,101 @@ QMatrix3x3 invert(const QMatrix3x3 &m)
 
 void CalibrationTool::checkIfFinished()
 {
-    if (m_calibratedTargets >= 4) {
-        Q_ASSERT(m_width > 0.0f);
-        Q_ASSERT(m_height > 0.0f);
+    if (m_state == State::Calibrating) {
+        if (m_calibratedTargets >= 4) {
+            Q_ASSERT(m_width > 0.0f);
+            Q_ASSERT(m_height > 0.0f);
 
-        // Calibration math based off of https://github.com/kreijack/xlibinput_calibrator
-        // The gist is that we want to calculate the difference between screen and touch position, and then normalize it before passing to KWin
+            // Calibration math based off of https://github.com/kreijack/xlibinput_calibrator
+            // The gist is that we want to calculate the difference between screen and touch position, and then normalize it before passing to KWin
 
-        enum {
-            UpperLeft = 0,
-            UpperRight = 1,
-            LowerLeft = 2,
-            LowerRight = 3,
-        };
-
-        // We have four points to use for calibration.
-        // To construct a mapping between screen space and touch space, we only need three points.
-        // So we use four sets of each permutation, and calculate an average based off of the sum of the individual matrices.
-        const std::array permutations{
-            std::array{UpperLeft, UpperRight, LowerLeft},
-            std::array{LowerRight, UpperRight, LowerLeft},
-            std::array{LowerRight, UpperLeft, LowerLeft},
-            std::array{LowerRight, UpperLeft, UpperRight},
-        };
-
-        QMatrix3x3 sum;
-        sum.fill(0.0f); // Ensure that the sum matrix is zeroed, not an identity. The identity will throw the sum off.
-
-        for (int i = 0; i < 4; i++) {
-            const QPointF screenA = m_screenPoints[permutations[i][0]];
-            const QPointF screenB = m_screenPoints[permutations[i][1]];
-            const QPointF screenC = m_screenPoints[permutations[i][2]];
-
-            // clang-format off
-            const std::array screenMatrix
-            {
-                static_cast<float>(screenA.x()), static_cast<float>(screenA.y()), 1.0f,
-                static_cast<float>(screenB.x()), static_cast<float>(screenB.y()), 1.0f,
-                static_cast<float>(screenC.x()), static_cast<float>(screenC.y()), 1.0f,
+            enum {
+                UpperLeft = 0,
+                UpperRight = 1,
+                LowerLeft = 2,
+                LowerRight = 3,
             };
-            // clang-format on
 
-            const QPointF touchA = m_touchPoints[permutations[i][0]];
-            const QPointF touchB = m_touchPoints[permutations[i][1]];
-            const QPointF touchC = m_touchPoints[permutations[i][2]];
-
-            // clang-format off
-            const std::array touchMatrix{
-                static_cast<float>(touchA.x()), static_cast<float>(touchA.y()), 1.0f,
-                static_cast<float>(touchB.x()), static_cast<float>(touchB.y()), 1.0f,
-                static_cast<float>(touchC.x()), static_cast<float>(touchC.y()), 1.0f
+            // We have four points to use for calibration.
+            // To construct a mapping between screen space and touch space, we only need three points.
+            // So we use four sets of each permutation, and calculate an average based off of the sum of the individual matrices.
+            const std::array permutations{
+                std::array{UpperLeft, UpperRight, LowerLeft},
+                std::array{LowerRight, UpperRight, LowerLeft},
+                std::array{LowerRight, UpperLeft, LowerLeft},
+                std::array{LowerRight, UpperLeft, UpperRight},
             };
-            // clang-format on
 
-            sum += invert(QMatrix3x3(touchMatrix.data())) * QMatrix3x3(screenMatrix.data());
+            QMatrix3x3 sum;
+            sum.fill(0.0f); // Ensure that the sum matrix is zeroed, not an identity. The identity will throw the sum off.
+
+            for (int i = 0; i < 4; i++) {
+                const QPointF screenA = m_screenPoints[permutations[i][0]];
+                const QPointF screenB = m_screenPoints[permutations[i][1]];
+                const QPointF screenC = m_screenPoints[permutations[i][2]];
+
+                // clang-format off
+                const std::array screenMatrix
+                {
+                    static_cast<float>(screenA.x()), static_cast<float>(screenA.y()), 1.0f,
+                    static_cast<float>(screenB.x()), static_cast<float>(screenB.y()), 1.0f,
+                    static_cast<float>(screenC.x()), static_cast<float>(screenC.y()), 1.0f,
+                };
+                // clang-format on
+
+                const QPointF touchA = m_touchPoints[permutations[i][0]];
+                const QPointF touchB = m_touchPoints[permutations[i][1]];
+                const QPointF touchC = m_touchPoints[permutations[i][2]];
+
+                // clang-format off
+                const std::array touchMatrix{
+                    static_cast<float>(touchA.x()), static_cast<float>(touchA.y()), 1.0f,
+                    static_cast<float>(touchB.x()), static_cast<float>(touchB.y()), 1.0f,
+                    static_cast<float>(touchC.x()), static_cast<float>(touchC.y()), 1.0f
+                };
+                // clang-format on
+
+                sum += invert(QMatrix3x3(touchMatrix.data())) * QMatrix3x3(screenMatrix.data());
+            }
+
+            QMatrix3x3 average = sum / 4.0f;
+
+            // Change and normalize coordinate spaces
+            average(0, 1) *= m_height / m_width;
+            average(0, 2) *= 1.0f / m_width;
+
+            average(1, 0) *= m_width / m_height;
+            average(1, 2) *= 1.0f / m_height;
+
+            // Remove the junk
+            average(2, 0) = 0.0f;
+            average(2, 1) = 0.0f;
+            average(2, 2) = 1.0f;
+
+            m_state = State::Confirming;
+            Q_EMIT stateChanged();
+            Q_EMIT calibrationCreated(QMatrix4x4(average));
+
+            // Reset the targets for re-confirmation
+            m_calibratedTargets = 0;
+            Q_EMIT currentTargetChanged();
+
+            // Start the countdown
+            m_resetTimer.start();
+            m_resetCountdown = 10;
+            Q_EMIT resetSecondsLeftChanged();
+
+            playSound(QStringLiteral("completion-success"));
         }
+    } else if (m_state == State::Confirming) {
+        if (m_calibratedTargets >= 4) {
+            m_resetTimer.stop();
 
-        QMatrix3x3 average = sum / 4.0f;
+            m_state = State::Testing;
+            Q_EMIT stateChanged();
 
-        // Change and normalize coordinate spaces
-        average(0, 1) *= m_height / m_width;
-        average(0, 2) *= 1.0f / m_width;
-
-        average(1, 0) *= m_width / m_height;
-        average(1, 2) *= 1.0f / m_height;
-
-        // Remove the junk
-        average(2, 0) = 0.0f;
-        average(2, 1) = 0.0f;
-        average(2, 2) = 1.0f;
-
-        m_finishedCalibration = true;
-        Q_EMIT finishedCalibrationChanged();
-        Q_EMIT finished(QMatrix4x4(average));
-
-        playSound(QStringLiteral("completion-success"));
+            playSound(QStringLiteral("completion-success"));
+        }
     }
 }
 
